@@ -123,7 +123,8 @@ def copy(source, target):
     target.__dict__ = source.__dict__
 
 
-def swap(root, replacement, name):
+def swap(body, replacement, name):
+    root = ast.Expression(body=body)
     for node in ast.walk(root):
         if (isinstance(node, ast.Name) and
             isinstance(node.ctx, ast.Load) and
@@ -393,6 +394,32 @@ class ASTCodeGenerator(object):
         self._write(' ' + self.binary_operators[node.op.__class__] + '= ')
         self.visit(node.value)
 
+    # JoinedStr(expr* values)
+    def visit_JoinedStr(self, node):
+        if node.values:
+            self._write('"".join((')
+            for value in node.values:
+                self.visit(value)
+                self._write(',')
+            self._write('))')
+        else:
+            self._write('""')
+
+    # FormattedValue(expr value)
+    def visit_FormattedValue(self, node):
+        if node.conversion == ord('r'):
+            self._write('repr')
+        elif node.conversion == ord('a'):
+            self._write('ascii')
+        else:
+            self._write('str')
+        self._write('(')
+        self.visit(node.value)
+        if node.format_spec is not None:
+            self._write(').__format__(')
+            self.visit(node.format_spec)
+        self._write(')')
+
     # Print(expr? dest, expr* values, bool nl)
     def visit_Print(self, node):
         self._new_line()
