@@ -216,8 +216,8 @@ emit_translate = template(is_func=True,
                           'default'),
                           func_defaults=(None,),
                           source=r"""
-    target = translate(msgid, default=default, domain=__i18n_domain, 
-                       context=__i18n_context, 
+    target = translate(msgid, default=default, domain=__i18n_domain,
+                       context=__i18n_context,
                        target_language=target_language)""")
 
 
@@ -949,7 +949,7 @@ class Compiler(object):
             module = ast.Module([])
             module.body += self.visit(node)
             ast.fix_missing_locations(module)
-            prelude = "__filename = %r" % filename
+            prelude = "__filename = %r\n__default = object()" % filename
             generator = TemplateCodeGenerator(module, source)
             tokens = [
                 Token(source[pos:pos + length], pos, source)
@@ -1306,8 +1306,8 @@ class Compiler(object):
 
         # Visit body to generate the message body
         code = self.visit(node.node)
-        swap(ast.Suite(body=code), load(append), "__append")
-        swap(ast.Suite(body=code), load(stream), "__stream")
+        swap(code, load(append), "__append")
+        swap(code, load(stream), "__stream")
         body += code
 
         # Reduce white space and assign as message id
@@ -1545,7 +1545,7 @@ class Compiler(object):
 
         # generate code
         code = self.visit(node.node)
-        swap(ast.Suite(body=code), load(append), "__append")
+        swap(code, load(append), "__append")
         body += code
 
         # output msgid
