@@ -26,6 +26,7 @@ from ..loader import TemplateLoader
 from ..astutil import Builtin
 from ..utils import decode_string
 from ..utils import string_type
+from ..utils import unicode_string
 
 from .program import MacroProgram
 
@@ -188,6 +189,8 @@ class PageTemplate(BaseTemplate):
 
     enable_data_attributes = False
 
+    enable_comment_interpolation = True
+
     on_error_handler = None
 
     restricted_namespace = True
@@ -208,7 +211,7 @@ class PageTemplate(BaseTemplate):
     @property
     def engine(self):
         if self.literal_false:
-            default_marker = ast.Str(s="__default__")
+            default_marker = Builtin("__default")
         else:
             default_marker = Builtin("False")
 
@@ -224,7 +227,7 @@ class PageTemplate(BaseTemplate):
 
     def parse(self, body):
         if self.literal_false:
-            default_marker = ast.Str(s="__default__")
+            default_marker = Builtin("__default")
         else:
             default_marker = Builtin("False")
 
@@ -237,6 +240,7 @@ class PageTemplate(BaseTemplate):
             implicit_i18n_attributes=self.implicit_i18n_attributes,
             trim_attribute_space=self.trim_attribute_space,
             enable_data_attributes=self.enable_data_attributes,
+            enable_comment_interpolation=self.enable_comment_interpolation,
             restricted_namespace=self.restricted_namespace,
             tokenizer=self.tokenizer
         )
@@ -310,7 +314,9 @@ class PageTemplate(BaseTemplate):
 
     def digest(self, body, names):
         hex = super(PageTemplate, self).digest(body, names)
-        digest = md5(hex.encode('ascii'))
+        if isinstance(hex, unicode_string):
+            hex = hex.encode('utf-8')
+        digest = md5(hex)
         digest.update(';'.join(names).encode('utf-8'))
 
         for attr in (
